package net.sf.buildbox.saxutil;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;

public class Raw {

    public static String toString(Object obj) {
        final Class objClass = obj.getClass();
        final StringBuilder sb = new StringBuilder(objClass.getSimpleName());
        sb.append('{');
        boolean first = true;
        for(Class c = objClass; c != null; c = c.getSuperclass()) {
            for (Field field : c.getDeclaredFields()) {
                if (first) {
                    first = false;
                } else {
                    sb.append(';');
                }
                if (Modifier.isPrivate(field.getModifiers())) continue;
                if (field.isSynthetic()) continue;
                sb.append(field.getName());
                sb.append(':');
                try {
                    field.setAccessible(true);
                    final Object value = field.get(obj);
                    if (value == null) {
                        sb.append("null");
                    } else if (field.getType().isArray()) {
                        sb.append(Arrays.asList((Object[])value));
                    } else if (value instanceof Date) {
                        sb.append(String.format("%tFT%1$tT.%1$tL", (Date) value));
                    } else if (value instanceof Calendar) {
                        sb.append(String.format("%tFT%1$tT.%1$tL", (Calendar) value));
                    } else {
                        sb.append(String.valueOf(value));
                    }
                } catch (IllegalAccessException e) {
                    sb.append("!(");
                    sb.append(e.getClass().getName());
                    sb.append(':');
                    sb.append(e.getMessage());
                    sb.append(')');
                }
            }
        }
        sb.append('}');
        return sb.toString();
    }

    public String toString() {
        return toString(this);
    }
}
