package net.sf.buildbox.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import org.codehaus.plexus.archiver.ArchiverException;
import org.codehaus.plexus.archiver.zip.ZipArchiver;
import org.codehaus.plexus.util.io.RawInputStreamFacade;

public class BbxZipUtils {

    public static void unzip(File srcZip, File destDirectory) throws IOException {
        final ZipFile zip = new ZipFile(srcZip);
        try {
            final Enumeration<? extends ZipEntry> zipEntries = zip.entries();
            while (zipEntries.hasMoreElements()) {
                final ZipEntry zipEntry = zipEntries.nextElement();
                if (zipEntry.isDirectory()) continue;
                final File f = new File(destDirectory, zipEntry.getName());
                f.getParentFile().mkdirs();
                if (f.exists()) {
                    throw new IOException("File already exists:" + f);
                }
                final InputStream is = zip.getInputStream(zipEntry);
                try {
                    org.codehaus.plexus.util.FileUtils.copyStreamToFile(new RawInputStreamFacade(is), f);
                    f.setLastModified(zipEntry.getTime());
                } finally {
                    is.close();
                }
            }
        } finally {
            zip.close();
        }
    }

    /**
     * @deprecated use {@link ZipArchiver} from library <code>org.codehaus.plexus:plexus-archiver</code> instead.
     * @param dir -
     * @param zipFile -
     * @throws IOException -
     */
    @Deprecated
    public static void zipDirectory(File dir, File zipFile) throws IOException {
        try {
            final ZipArchiver za = new ZipArchiver();
            za.setCompress(false);
            za.addDirectory(dir);
            za.setDestFile(zipFile);
            za.createArchive();
        } catch (ArchiverException e) {
            throw new IOException(e.getClass().getName() + ": " + e.getMessage());
        }
    }

    /**
     * @deprecated use {@link ZipArchiver} from library <code>org.codehaus.plexus:plexus-archiver</code> instead.
     * @param dir -
     * @param zipFile -
     * @throws IOException -
     */
    @Deprecated
    public static void zipDirectory(File dir, Collection<String> explicitFiles, File zipFile) throws IOException {
        try {
            final ZipArchiver za = new ZipArchiver();
            za.setCompress(true);
            za.addDirectory(dir, explicitFiles.toArray(new String[explicitFiles.size()]), null);
            za.setDestFile(zipFile);
            za.createArchive();
        } catch (ArchiverException e) {
            throw new IOException(e.getClass().getName() + ": " + e.getMessage());
        }
    }
}
