package net.sf.buildbox.util;

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.TimeUnit;
import java.io.File;
import java.io.IOException;

public class ExtFileLock implements Lock {
    private long timequantum = 250;
    private final boolean exclusive;
    private final File path;
    private final File lockdir;
    private final File lockfileExclusive;
    private File lockfile;

    public ExtFileLock(boolean exclusive, File path) {
        this.exclusive = exclusive;
        this.path = path;
        this.lockdir = new File(path + ".lock");
        lockfileExclusive = new File(lockdir, "EXCLUSIVE");
    }

    public void lock() {
        while (true) {
            try {
                lockInterruptibly();
                return;
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    public void lockInterruptibly() throws InterruptedException {
        while (! tryLock()) {
            Thread.sleep(timequantum, 0);
        }
    }

    public boolean tryLock(long time, TimeUnit unit) throws InterruptedException {
        final long millis;
        switch (unit) {
/* java 1.6 only:
        case DAYS:
            millis = time * 24 * 60 * 60 * 1000;
            break;
        case HOURS:
            millis = time * 60 * 60 * 1000;
            break;
        case MINUTES:
            millis = time * 60000;
            break;
*/
        case NANOSECONDS:
            millis = time / 1000000;
            break;
        case MICROSECONDS:
            millis = time / 1000;
            break;
        case MILLISECONDS:
            millis = time;
            break;
        case SECONDS:
            millis = time * 1000;
            break;
        default:
            throw new IllegalStateException("unit=" + unit);
        }
        long endTime = System.currentTimeMillis() + millis;
        while (! tryLock()) {
            long delay = endTime - System.currentTimeMillis();
            if (delay > timequantum) {
                delay = timequantum;
            }
            if (delay < 1) {
                return false;
            }
            Thread.sleep(timequantum, 0);
        }
        return true;
    }

    public boolean tryLock() {
        if (exclusive) {
            if (!lockdir.mkdirs()) return false;
            lockfile = lockfileExclusive;
        } else {
            if (lockfileExclusive.exists()) return false;
            try {
                lockdir.mkdirs();
                lockfile = File.createTempFile("shared.",".lock", lockdir);
            } catch (IOException e) {
                e.printStackTrace();
                unlock();
                return false;
            }
        }
        try {
            SimpleTextFile.simpleSave(lockfile, "Exclusive:" + exclusive);
        } catch (IOException e) {
            e.printStackTrace();
            unlock();
            return false;
        }
        return true;
    }

    public void unlock() {
        if (lockfile != null) {
            if (!lockfile.delete()) {
//                System.err.println("deleteOnExit: lockfile " + lockfile);
                lockfile.deleteOnExit();
            }
        }
        if (!lockdir.delete()) {
//            System.err.println("deleteOnExit: lockdir " + lockdir);
            lockdir.deleteOnExit();
        }
    }

    public Condition newCondition() {
        throw new UnsupportedOperationException();
    }
}
