package net.sf.buildbox.util;

import java.util.concurrent.Callable;

/**
 * Executes a task and monitors its execution in separate thread.
 * @param <T>
 */
public abstract class SlowTaskMon<T> implements Callable<T> {
    private final Callable<T> callable;
    protected final String actionName;
    private final long firstPeriod;
    private final long nextPeriod;
    private Object so;

    public SlowTaskMon(String actionName, long period, Callable<T> callable) {
        this(actionName, period, period, callable);
    }

    public SlowTaskMon(String actionName, long firstPeriod, long nextPeriod, Callable<T> callable) {
        this.callable = callable;
        this.actionName = actionName;
        this.firstPeriod = firstPeriod;
        this.nextPeriod = nextPeriod;

    }

    public synchronized final T call() throws Exception {
        final Runnable mon = new Runnable() {
            public void run() {
                try {
                    onStarting();
                    synchronized (so) {
                        so.wait(firstPeriod);
                        while (so != null) {
                            stillWorking();
                            so.wait(nextPeriod);
                        }
                    }
                    onCompleted();
                } catch (InterruptedException e) {
                    onInterrupted();
                }
            }
        };
        final Thread monThread = new Thread(mon, actionName + ".mon");
        try {
            so = callable;
            monThread.start();
            return callable.call();
        } finally {
            synchronized (so) {
                final Object soBak = so;
                so = null; // requests to stop monitoring
                soBak.notify();
            }
            monThread.join();
        }
    }

    protected void onStarting() {
        //
    }

    public abstract void stillWorking();

    protected void onCompleted() {
        //
    }

    protected void onInterrupted() {
        System.err.println("interrupted: " + actionName);
        Thread.currentThread().interrupt();
    }
}
