package net.sf.buildbox.util;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * @deprecated use {@link BbxSystemUtils} instead
 */
@Deprecated
public final class SystemUtils {
    private SystemUtils() {
        // not instantiable
    }

    /**
     * @deprecated use {@link net.sf.buildbox.util.CommandLineExec} instead.
     */
    @Deprecated
    public static int execute(File dir, String[] cmdArray, final String[] envp, final OutputEater eater) throws IOException {
        final LinkedList<String> args = new LinkedList<String>(Arrays.asList(cmdArray));
//        final List<String> args = Arrays.asList(cmdArray); //TODO: using this line instead of the one above causes hanging in SvnInfoReaderTest. The line is incorrect, however such a behavior is suspicious and deserves to be explored.
        final CommandLineExec exec = new CommandLineExec<Object>(args.remove(0)) {
            protected void overrideEnv(Map<String, String> env) {
                if (envp != null) {
                    throw new UnsupportedOperationException("not yet implemented, use CommandLineExec instead");
                }
            }

            protected void log(String message) {
                eater.info(message);
            }

            protected void stdout(String line) {
                eater.stdout(line);
            }

            protected void stderr(String line) {
                eater.stderr(line);
            }
        };
        exec.setBasedir(dir == null ? new File(System.getProperty("user.dir")) : dir);
        exec.addArguments(args);
        exec.setFailOnError(false);
        try {
            return exec.call();
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            return 111;
        } finally {
            if (eater != null) {
                eater.close();
            }
        }
    }

    public static int executeJava(File dir, File javaHome, List<File> classPath, List<String> jvmOptions, String mainClass, List<String> arguments, OutputEater eater) throws IOException {
        final List<String> cmds = new LinkedList<String>();
        final String java = javaHome == null ? "java" : new File(javaHome, "bin/java").getAbsolutePath();
        cmds.add(java);
        if (classPath != null && ! classPath.isEmpty()) {
            cmds.add("-cp");
            cmds.add(BbxSystemUtils.createClasspathString(classPath));
        }
        if (jvmOptions != null) {
            cmds.addAll(jvmOptions);
        }
        cmds.add(mainClass);
        if (arguments != null) {
            cmds.addAll(arguments);
        }
        return execute(dir, cmds.toArray(new String[cmds.size()]), null, eater);
    }

    public static interface OutputEater {
        public void stdout(String line);
        public void stderr(String line);
        public void info(String line);
        public void close();
    }

}
