package net.sf.buildbox.saxutil;

import java.lang.reflect.*;
import java.util.*;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * This class is written in an extremely dirty way.
 * It's just a proof of concept, beautification will hopefully happen soon.
 * @deprecated use XmlBeans or xstream or something like that instead
 */
@Deprecated
public class ReflectiveSaxHandler<T> extends DefaultHandler {

    private final InitialContext initialContext;
    private Context currentContext;

    public <T> ReflectiveSaxHandler(Class<T> pojoClass, String rootElement) {
        initialContext = new InitialContext(pojoClass, rootElement);
        currentContext = initialContext;
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        final ParentContext current = (ParentContext) currentContext;
        try {
            currentContext = current.createContext(qName);
            currentContext.setAttributes(attributes);
        } catch (NoSuchFieldException e) {
            throw new SAXException(getContextPath(), e);
        } catch (SAXException e) {
            throw new SAXException(getContextPath(), e);
        }
    }

    private String getContextPath() {
        String result = "";
        Context cc = currentContext.getParentContext();
        while (cc != null) {
            result = "/" + cc.toString() + result;
            cc = cc.getParentContext();
        }
        return result;
    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
        currentContext = currentContext.close();
    }

    public void characters(char ch[], int start, int length) throws SAXException {
        currentContext.readChunk(ch, start, length);
    }

    @SuppressWarnings("unchecked")
    public T getResult() {
        return (T) initialContext.getValue();
    }

    private static abstract class Context {
        private ParentContext parentContext;

        protected Context(ParentContext parentContext) {
            this.parentContext = parentContext;
        }

        abstract void readChunk(char[] ch, int start, int length);

        abstract ParentContext close() throws SAXException;

        ParentContext getParentContext() {
            return parentContext;
        }

        public abstract void setAttributes(Attributes attributes) throws SAXException;
    }

    private static abstract class ParentContext extends Context {
        protected ParentContext(ParentContext parentContext) {
            super(parentContext);
        }

        abstract Context createContext(String elementName) throws NoSuchFieldException, SAXException;

        abstract void addValue(Object value) throws SAXException;

        public abstract String toString();
    }

    private static class IgnoringContext extends ParentContext {
        protected IgnoringContext(ParentContext parentContext) {
            super(parentContext);
        }

        public void readChunk(char[] ch, int start, int length) {
            // ignored
        }

        ParentContext close() throws SAXException {
            return getParentContext();
        }

        public void setAttributes(Attributes attributes) {
            // ignored
        }

        /**
         * Posts the value to its parent context
         * @param value  the value to add to parent's current field
         */
        public void addValue(Object value) {
            // ignored
        }

        public String toString() {
            return "*";
        }

        Context createContext(String elementName) throws NoSuchFieldException, SAXException {
            return new IgnoringContext(this);
        }
    }

    private static class StringContext extends Context {
        private StringBuilder stringBuilder = new StringBuilder();

        protected StringContext(ParentContext parentContext) {
            super(parentContext);
        }

        public void readChunk(char[] ch, int start, int length) {
            stringBuilder.append(ch, start, length);
        }

        ParentContext close() throws SAXException {
            getParentContext().addValue(stringBuilder.toString());
            return getParentContext();
        }

        public void setAttributes(Attributes attributes) {
            //
        }

    }

    private static class InitialContext extends ParentContext {
        private final Class rootClass;
        private final String rootElement;
        private Object value;

        protected InitialContext(Class rootClass, String rootElement) {
            super(null);
            this.rootClass = rootClass;
            this.rootElement = rootElement;
        }

        Context createContext(String elementName) throws NoSuchFieldException, SAXException {
            if (! elementName.equals(rootElement)) {
                throw new NoSuchFieldException(elementName);
            }
            try {
                final Constructor con = rootClass.getDeclaredConstructor();
                con.setAccessible(true);
                return new StructContext(this, con.newInstance());
            } catch (InstantiationException e) {
                throw new SAXException(elementName, e);
            } catch (IllegalAccessException e) {
                throw new SAXException(elementName, e);
            } catch (NoSuchMethodException e) {
                throw new SAXException(elementName, e);
            } catch (InvocationTargetException e) {
                throw new SAXException(elementName, e);
            }
        }

        void addValue(Object value) throws SAXException {
            this.value = value;
        }

        void readChunk(char[] ch, int start, int length) {
            //TODO
        }

        ParentContext close() throws SAXException {
            return null;
        }

        public void setAttributes(Attributes attributes) {
            //
        }

        public Object getValue() {
            return value;
        }

        public String toString() {
            return this.rootElement;
        }
    }

    private static Context dispatch(ParentContext parentContext, Class fieldType) throws SAXException {
        //TODO: check for supported types reflectively, on a transformer class with many toString(XXX) methods
        if (String.class.equals(fieldType)) {
            return new StringContext(parentContext);
        } else if (boolean.class.equals(fieldType)) {
            return new StringContext(parentContext);
        } else if (Boolean.class.equals(fieldType)) {
            return new StringContext(parentContext);
        } else if (Enum.class.isAssignableFrom(fieldType)) {
            return new StringContext(parentContext);
        }
        // probably a complex type, create a new context for it
        try {
            final Constructor fieldValueConstructor = fieldType.getDeclaredConstructor();
            fieldValueConstructor.setAccessible(true);
            final Object fieldValue = fieldValueConstructor.newInstance();
            return new StructContext(parentContext, fieldValue);
        } catch (InstantiationException e) {
            e.printStackTrace();
            throw new SAXException(fieldType.getName(), e);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
            throw new SAXException(fieldType.getName(), e);
        } catch (InvocationTargetException e) {
            throw new SAXException(fieldType.getName(), e);
        } catch (NoSuchMethodException e) {
            throw new SAXException(fieldType.getName(), e);
        }
    }


    private static class ItemContext extends ParentContext {
        private final String itemElementName;
        private final Class itemClass;
        private final Collection<Object> collection = new LinkedList<Object>();

        protected ItemContext(ParentContext parentContext, Field collectionField) {
            super(parentContext);
            CollInfo collInfo = collectionField.getAnnotation(CollInfo.class);
            itemElementName = collInfo.item();
            itemClass = collInfo.itemClass();
        }

        Context createContext(String elementName) throws NoSuchFieldException, SAXException {
            if (!itemElementName.equals(elementName)) {
                throw new NoSuchFieldException(elementName);
            }
            return dispatch(this, itemClass);
        }

        void addValue(Object value) throws SAXException {
            collection.add(value);
        }

        void readChunk(char[] ch, int start, int length) {
            //TODO
        }

        ParentContext close() throws SAXException {
            getParentContext().addValue(collection); //todo check for exact type
            return getParentContext();
        }

        public void setAttributes(Attributes attributes) {
            //
        }

        public String toString() {
            return itemElementName + "[" + (collection.size()+1) + "]";
        }
    }

    private static class PropertiesContext extends ParentContext {
        private final Map<String, String> values = new LinkedHashMap<String, String>();
        private String currentKey;

        protected PropertiesContext(ParentContext parentContext) {
            super(parentContext);
        }

        Context createContext(String elementName) throws NoSuchFieldException, SAXException {
            this.currentKey = elementName;
            return new ConfigurationPropertyContext(this); //TODO: DIRTY - we should use PropertyContext here and make separate container for ConfigurationPropertyContext
        }

        void addValue(Object value) throws SAXException {
            values.put(currentKey, (String) value);
        }

        public String toString() {
            return currentKey + "[]";
        }

        void readChunk(char[] ch, int start, int length) {
            //
        }

        ParentContext close() throws SAXException {
            getParentContext().addValue(values);
            return getParentContext();
        }

        public void setAttributes(Attributes attributes) throws SAXException {
            //
        }
    }

    /**
     * Reads string property values only. Though unused, I left it here to have easier way back.
     */
    private static class PropertyContext extends Context {
        private final StringBuilder sb = new StringBuilder();
        protected PropertyContext(ParentContext parentContext) {
            super(parentContext);
        }

        void readChunk(char[] ch, int start, int length) {
            sb.append(ch, start, length);
        }

        ParentContext close() throws SAXException {
            final PropertiesContext parent = (PropertiesContext) getParentContext();
            parent.addValue(sb.toString());
            return getParentContext();
        }

        public void setAttributes(Attributes attributes) throws SAXException {
            //
        }
    }

    /**
     * Basically same as {@link net.sf.buildbox.saxutil.ReflectiveSaxHandler.PropertyContext} but ignores structured values rather than failing on them.
     */
    private static class ConfigurationPropertyContext extends ParentContext {
        private final StringBuilder sb = new StringBuilder();
        protected ConfigurationPropertyContext(ParentContext parentContext) {
            super(parentContext);
        }

        Context createContext(String elementName) throws NoSuchFieldException, SAXException {
            return new IgnoringContext(this);
        }

        void addValue(Object value) throws SAXException {
            // ignored
            // TODO: after all, perhaps we could build the tree of objects here ?
        }

        public String toString() {
            return "?";
        }

        void readChunk(char[] ch, int start, int length) {
            sb.append(ch, start, length);
        }

        ParentContext close() throws SAXException {
            final PropertiesContext parent = (PropertiesContext) getParentContext();
            parent.addValue(sb.toString());
            return parent;
        }

        public void setAttributes(Attributes attributes) throws SAXException {
            //
        }
    }

    private static class StructContext extends ParentContext {
        private final Object targetObject;
        private final Class<?> targetClass;

        private Map<Field, Collection<Object>> arrayCollectors = new HashMap<Field, Collection<Object>>();

        /**
         * When reading text content, this indicates where will the text go after reading finishes.
         * For non-{@link String} fields, the necessary conversion will be attempted.
         */
        private Field currentField;

        protected StructContext(ParentContext parentContext, Object targetObject) {
            super(parentContext);
            this.targetObject = targetObject;
            this.targetClass = targetObject.getClass();
        }

        private static Field getDeclaredFieldRecursive(Class<?> theClass, String fieldName) throws NoSuchFieldException {
            final Class<?> origClass = theClass;
            while (true) {
                try {
                    return theClass.getDeclaredField(fieldName);
                } catch (NoSuchFieldException e) {
                    theClass = theClass.getSuperclass();
                    if (theClass == null) throw new NoSuchFieldException(origClass.getName() + "." + fieldName);
                }
            }
        }

        public Context createContext(String fieldName) throws NoSuchFieldException, SAXException {
            currentField = getDeclaredFieldRecursive(targetClass, fieldName.replace('-','_'));
            currentField.setAccessible(true);
            final Class fieldType = currentField.getType();
            if (Object.class.equals(fieldType)) {
                return new IgnoringContext(this);
            }
            // handle arrays as unwrapped
            if (Object[].class.isAssignableFrom(fieldType)) {
                Collection<Object> coll = arrayCollectors.get(currentField);
                if (coll == null) {
                    coll = new LinkedList<Object>();
                    arrayCollectors.put(currentField, coll);
                }
                return dispatch(this, fieldType.getComponentType());
            }
            // handle wrapped collections (at least List)
            if (Collection.class.isAssignableFrom(fieldType)) {
                return new ItemContext(this, currentField);
            }
            // property map - elem. name=key, content=value
            if (Map.class.isAssignableFrom(fieldType)) {
                return new PropertiesContext(this);
            }
            return dispatch(this, fieldType);

        }

        void addValue(Object value) throws SAXException {
            Collection<Object> coll = arrayCollectors.get(currentField);
            if (coll != null) {
                coll.add(value);
            } else try {
                final Class<?> fieldType = currentField.getType();
//                boolean isSimpleType = Enum.class.isAssignableFrom(fieldType) ;
                boolean isSimpleType = value instanceof String;
                if (isSimpleType && !fieldType.equals(String.class)) {
                    // lookup convertor from string to desired type
                    try {
                        // try static valueOf(String) method
                        Method m = fieldType.getMethod("valueOf", String.class);
                        m.setAccessible(true);
                        value = m.invoke(null, value);
                    } catch (NoSuchMethodException e) {
                        //todo: try string constructor
                        throw new SAXException("No way to convert this type to string: " + fieldType, e);
                    }
                }
                currentField.set(targetObject, value);
                currentField = null;
            } catch (IllegalAccessException e) {
                throw new SAXException(currentField.getName(), e);
            } catch (InvocationTargetException e) {
                if (e.getCause() instanceof Exception) {
                    throw new SAXException(currentField.getName(), (Exception) e.getCause());
                } else {
                    e.printStackTrace();
                    throw new SAXException(currentField.getName(), e);
                }
            }
        }

        public ParentContext close() throws SAXException {
            for (Map.Entry<Field, Collection<Object>> entry : arrayCollectors.entrySet()) {
                try {
                    final Field field = entry.getKey();
//                    System.out.println("field: " + field);
                    final Object arrayValue = collectionToArray(entry.getValue(), field.getType().getComponentType());
//                    System.out.println("arrayValue = " + arrayValue);
                    field.setAccessible(true);
                    field.set(targetObject, arrayValue);
                } catch (IllegalAccessException e) {
                    throw new SAXException(e);
                }
            }
            getParentContext().addValue(targetObject);
            return getParentContext();
        }

        public void setAttributes(Attributes attributes) throws SAXException {
            for (int i = 0; i<attributes.getLength(); i++) {
                final String attrName = attributes.getLocalName(i);
                if (attrName.equals("xmlns")) continue;
                if (attrName.startsWith("xmlns:")) continue;
                final String nsUri = attributes.getURI(i);
                if (nsUri != null && !nsUri.equals("")) continue;
                final String attrValue = attributes.getValue(i);
                try {
                    final Field field = getDeclaredFieldRecursive(targetClass, attrName.replace('-','_'));
                    field.setAccessible(true);
                    //TODO: support also non-string types
                    field.set(targetObject, attrValue);
                } catch (NoSuchFieldException e) {
                    throw new SAXException(targetClass.getName(), e);
                } catch (IllegalAccessException e) {
                    throw new SAXException(targetClass.getName(), e);
                }
            }
        }

        private Object collectionToArray(Collection<?> value, Class componentType) {
            final Object sampleArray = Array.newInstance(componentType, 0);
            return value.toArray((Object[]) sampleArray);
        }

        public void readChunk(char[] ch, int start, int length) {
            // ignored todo throw something ?
        }

        public String toString() {
            return currentField == null ? "null" : currentField.getName();
        }
    }
}
